@@ -85,38 +85,38 @@ class DataContainer:
     # Note: None of these functions are allowed to *modify* the dicts, instead they must create new dicts
     # This avoids requring deep copies for everything
 
-    def apply_for_notes(self, f, *args, types: list = NOTE_TYPES, **kwargs) -> None:
+    def apply_for_notes(self, f, *args, types: list = NOTE_TYPES, mirror_left: bool = False, **kwargs) -> None:
         for t in types:
             if t not in NOTE_TYPES:
                 continue
             notes = getattr(self, t)
             out = {}
             for _, nodes in sorted(notes.items()):
-                out_nodes = f(nodes, *args, **kwargs)
+                out_nodes = f(nodes, *args, direction=(-1 if mirror_left and t == "left" else 1), **kwargs)
                 out[out_nodes[0, 2]] = out_nodes
             setattr(self, str(t), out)
 
-    def apply_for_walls(self, f, *args, types: list = WALL_TYPES, **kwargs) -> None:
+    def apply_for_walls(self, f, *args, types: list = WALL_TYPES, mirror_left: bool = False, **kwargs) -> None:
         wall_types = [WALL_TYPES[t][0] for t in types if t in WALL_TYPES]
         out_walls = {}
         for time_index, wall in sorted(self.walls.items()):
             if wall[0, 3] in wall_types:
-                wall = f(wall, *args, **kwargs)
+                wall = f(wall, *args, **kwargs)  # TODO: support mirror_left
                 out_walls[wall[0, 2]] = wall
             else:
                 out_walls[time_index] = wall
         self.walls = out_walls
 
-    def apply_for_all(self, f, *args, types: list = ALL_TYPES, **kwargs) -> None:
-        self.apply_for_notes(f, *args, types=types, **kwargs)
-        self.apply_for_walls(f, *args, types=types, **kwargs)
+    def apply_for_all(self, f, *args, types: list = ALL_TYPES, mirror_left: bool = False, **kwargs) -> None:
+        self.apply_for_notes(f, *args, types=types, mirror_left=mirror_left, **kwargs)
+        self.apply_for_walls(f, *args, types=types, mirror_left=mirror_left, **kwargs)
 
     # used when the functions needs access to all notes and rails of a color at one
-    def apply_for_note_types(self, f, *args, types: list = NOTE_TYPES, **kwargs) -> None:
+    def apply_for_note_types(self, f, *args, types: list = NOTE_TYPES, mirror_left: bool = False, **kwargs) -> None:
         for t in types:
             if t not in NOTE_TYPES:
                 continue
-            setattr(self, t, f(getattr(self, t), *args, **kwargs))
+            setattr(self, t, f(getattr(self, t), *args, direction=(-1 if mirror_left and t == "left" else 1), **kwargs))
 
     def filtered(self, types: list = ALL_TYPES) -> "DataContainer":
         replacement = {
@@ -138,6 +138,16 @@ class DataContainer:
             setattr(self, t, self_notes | other_notes)
         self.walls |= other.walls
 
+    def get_object_dict(self, type_name: str) -> SINGLE_COLOR_NOTES | WALLS:
+        if type_name in NOTE_TYPES:
+            return getattr(self, type_name)
+        wall_type = WALL_TYPES[type_name][0]
+        return {
+            time_index: wall
+            for time_index, wall in sorted(self.walls.items())
+            if wall[0, 3] == wall_type
+        }
+
 @dataclasses.dataclass
 class ClipboardDataContainer(DataContainer):
     original_json: str
@@ -369,4 +379,4 @@ def export_clipboard(data: DataContainer, realign_start: bool = True):
 def import_file(file_path: Path) -> SynthFile:
     out = SynthFile(file_path, {}, {}, {})
     out.reload()
-    return out
\ No newline at end of file
+    return out
